// -*- C++ -*-

//
//  Xlpq frontend to lpq,lprm,lpc
//
//  Copyright (C) 1996 Christoph Neerfeld
//  email:  Christoph.Neerfeld@home.ivm.de
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

extern "C" {
#include <forms.h>
#include "printer.h"
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <sys/time.h>
}

#include <iostream.h>
#include <fstream.h>
#include <String.h>

#include "config.h"

#define BGCOL_DEFAULT {160, 176, 192}	// default form-background color
#define BGCOL_STRING  "#90A0B0"

FD_fr1 *fr1;

typedef struct {	// the color structure
  int r, g, b;
} color_t;

char bgcols[16+1] = {0};		// the background color string
color_t bgcol = BGCOL_DEFAULT;		// default background for all forms
int winPosX,				// hint to WM for position
    winPosY;
unsigned int winWidth,			// request for different size
	     winHeight;
int help = 0;				// set if cmd line help requested
char printerName[32+1] = {0};		// printer name string
char fileName[512+1] = {0};		// name of file to print
char geoms[32+1] = {0};			// the geometry string
int placementMethod = FL_PLACE_FREE;	// top form placement method

FL_CMD_OPT options[] = 
{
  {"-h", ".help", XrmoptionNoArg, "True"},
  {"-P", ".printerName", XrmoptionSepArg, ""},
  {"-send", ".fileName", XrmoptionSepArg, ""},
  {"-bgcol", ".bgcol", XrmoptionSepArg, BGCOL_STRING},
  {"-geometry", ".geometry", XrmoptionSepArg, ""}
};

FL_resource resources[] =
{
  {"help", "Help", FL_BOOL, &help, "False"},
  {"printerName", "PrinterName", FL_STRING, printerName, "", 32},
  {"fileName", "FileName", FL_STRING, fileName, "", 512},
  {"bgcol", "BGColor", FL_STRING, bgcols, BGCOL_STRING, 16},
  {"geometry", "Geometry", FL_STRING, geoms, "", 32}
};


void f_select (FL_OBJECT *ob, long user_data)
{
  f_update( (FL_OBJECT *) 0, 0);
}

void f_queue (FL_OBJECT *ob, long user_data)
{
  FILE *file;
  String progname;
  if(fl_get_button( ob))
    progname = (String) LPC + " enable " + (String) fl_get_choice_text(fr1->choice_1);
  else
    progname = (String) LPC + " disable " + (String) fl_get_choice_text(fr1->choice_1);
  file = popen((const char *) progname, "r");
  unsigned char c;
  String temp;
  while(c = fgetc(file))
    {
      if ( c >= 128) break;
      if ( c != '\n' )
	temp += c;
      else
	{
	  if(temp.contains("Privileged"))
	    {
	      fl_show_alert("Permission denied.", "This is a privileged command", \
			    "and you do not have permission to execute it.", 0);
	      pclose(file);
	      if(fl_get_button(ob))
		fl_set_button(ob, 0);
	      else
		fl_set_button(ob, 1);
	      return;
	    }
	  temp = "";
	}
    }
  pclose(file);
  f_update( (FL_OBJECT *) 0, 0);
  fl_select_browser_line(fr1->browser_1, 1);
}

void f_print (FL_OBJECT *ob, long user_data)
{
  FILE *file;
  String progname;
  if(fl_get_button( ob))
    progname = (String) LPC + " start " + (String) fl_get_choice_text(fr1->choice_1);
  else
    progname = (String) LPC + " stop " + (String) fl_get_choice_text(fr1->choice_1);
  file = popen((const char *) progname, "r");
  unsigned char c;
  String temp;
  while(c = fgetc(file))
    {
      if ( c >= 128) break;
      if ( c != '\n' )
	temp += c;
      else
	{
	  if(temp.contains("Privileged"))
	    {
	      fl_show_alert("Permission denied.", "This is a privileged command", \
			    "and you do not have permission to execute it.", 0);
	      pclose(file);
	      if(fl_get_button(ob))
		fl_set_button(ob, 0);
	      else
		fl_set_button(ob, 1);
	      return;
	    }
	  temp = "";
	}
    }
  pclose(file);
  f_update( (FL_OBJECT *) 0, 0);
  fl_select_browser_line(fr1->browser_1, 1);
}

void f_remove (FL_OBJECT *ob, long user_data)
{
  String words[3];
  char msg[256];
  int i = fl_get_browser(fr1->browser_1);
  if (i == 0) return;
  String line = fl_get_browser_line(fr1->browser_1, i);
  split(line, words, 3, RXwhite);
  sprintf(msg, "Do you realy want to remove job Nr. %s", (const char*) words[2]);
  if(fl_show_question((const char *)msg, 0))
    {
      String progname;
      progname = (String) "lprm -P " + (String) fl_get_choice_text(fr1->choice_1) + \
	" " +(String) words[2];
      system((const char *) progname);
      f_update( (FL_OBJECT *) 0, 0);
      fl_select_browser_line(fr1->browser_1, 1);
    }
}

void f_make_top (FL_OBJECT *ob, long user_data)
{
  FILE *file;
  String words[3];
  char msg[256];
  int i = fl_get_browser(fr1->browser_1);
  if ( i == 0) return;
  String line = fl_get_browser_line(fr1->browser_1, i);
  split(line, words, 3, RXwhite);
  sprintf(msg, "Do you realy want to move job Nr. %s on top of queue", \
		  (const char*) words[2]);
  if(fl_show_question((const char *)msg, 0))
    {
      String progname;
      progname = (String) LPC + " topq " + (String) fl_get_choice_text(fr1->choice_1) + \
	" " +(String) words[2];
      file = popen((const char *) progname, "r");
      unsigned char c;
      String temp;
      while(c = fgetc(file))
	{
	  if ( c >= 128) break;
	  if ( c != '\n' )
	    temp += c;
	  else
	    {
	      if(temp.contains("Privileged"))
		{
		  fl_show_alert("Permission denied.", "This is a privileged command", \
				"and you do not have permission to execute it.", 0);
		  pclose(file);
		  return;
		}
	      temp = "";
	    }
	}
      pclose(file);
      f_update( (FL_OBJECT *) 0, 0);
      fl_select_browser_line(fr1->browser_1, 1);
    }
}

void f_browser (FL_OBJECT *ob, long user_data)
{

}

void f_update (FL_OBJECT *ob, long user_data)
{
  FILE *file;
  String progname;
  progname = (String) "lpq -P " + (String) fl_get_choice_text(fr1->choice_1);
  file = popen((const char *) progname, "r");
  unsigned char c;
  String temp, label;
  label = "";
  short i=0;
  fl_clear_browser(fr1->browser_1);
  while(c = fgetc(file))
    {
      if ( c >= 128) break;
      if ( c != '\n' )
	temp += c;
      else
	{
	  if(temp.contains("Rank"))
	     { i = 1; temp = ""; continue; }
	  if(i == 0) 
	    {
	      label += temp + '\n';
	      fl_set_object_label(fr1->status, (const char*) label);
	    }
	  else
	    {
	      fl_add_browser_line(fr1->browser_1, (const char*) temp);
	    }
	  temp = "";
	}
    }
  pclose(file);
  // --- exec lpc
  progname = (String) LPC + " status " + (String) fl_get_choice_text(fr1->choice_1);
  file = popen((const char *) progname, "r");
  while(c = fgetc(file))
    {
      if ( c >= 128) break;
      if ( c != '\n' )
	temp += c;
      else
	{
	  if(temp.contains("queuing"))
	     { 
	       if(temp.contains("disabled"))
		 { fl_set_button(fr1->queuing, 0); }
	       else
		 { fl_set_button(fr1->queuing, 1); }
	       temp = ""; continue; 
	     }
	  else if (temp.contains("printing"))
	    { 
	       if(temp.contains("disabled"))
		 { fl_set_button(fr1->printing, 0); }
	       else
		 { fl_set_button(fr1->printing, 1); }
	       temp = ""; continue; 
	     }
	  temp = "";
	}
    }
  pclose(file);
  fl_select_browser_line(fr1->browser_1, 1);
  fl_redraw_object(fr1->status_frame);
  return;
}

void f_quit (FL_OBJECT *ob, long user_data)
{
  fl_finish();
  exit(0);
}

void f_timer ( int sig, void *data)
{
  f_update( (FL_OBJECT *) 0, 0);
  alarm(5);
}

void f_auto (FL_OBJECT *ob, long user_data)
{
  if(fl_get_button(ob))
    { fl_add_signal_callback(SIGALRM, f_timer, (void *) 0); alarm(5); }
  else
    { alarm (0); fl_remove_signal_callback(SIGALRM); }
}

void f_version (FL_OBJECT *ob, long user_data)
{
  fl_show_message("This is Xlpq Version 1.0.2 by Christoph Neerfeld",\
		  "This program is copyrighted, but you may distribute it",\
		  "under the terms of the GNU General Public Licence.");
}

void colorSqueeze(void)
{
  int i;

  for (i=0; i<FL_FREE_COL1; i++) {
    switch (i) {

      case FL_BLACK:		// except for these which
      case FL_YELLOW:		// are used in our code
      case FL_INDIANRED:
      case FL_COL1:
      case FL_RIGHT_BCOL:
      case FL_BOTTOM_BCOL:
      case FL_TOP_BCOL:
      case FL_LEFT_BCOL:
      case FL_MCOL:
      case FL_INACTIVE:
      case FL_GREEN:
	break;

      default:
	fl_set_icm_color(i, 0,0,0);	// reset all unused internal
    }					// colormap colors to black
  }
}

// Parse user-specified background color from string

void bgColor(char *color)
{
  color_t bgcol_default = BGCOL_DEFAULT;
  int i;

  i = sscanf(color, "#%2X%2X%2X",		// scan the hex color
             &bgcol.r, &bgcol.g, &bgcol.b);
  if (i != 3)					// if scan unsuccessful
    bgcol = bgcol_default;			// use the default color
  fl_mapcolor(FL_INDIANRED,			// replace the one used
    bgcol.r, bgcol.g, bgcol.b);
}

// Process arguments parsed in resources

void processArgs(void)
{
  int status;

  if (help) {
    cout << "usage: xlpq [ [-h] [-P printername] [-send file] "
	    "[-bgcol #<6-digit hex color>]\n"
	    "              [-geometry [=][<W>{xX}<H>][{+-}<XO>{+-}<YO>]] ]" \
         << endl;
    exit(1);
  }
  bgColor(bgcols);				// parse user bg color
  status = XParseGeometry(geoms, &winPosX,	// parse geometry from
			  &winPosY, &winWidth,	// the input string
			  &winHeight);
  if ((status & XValue)&&(status && YValue)) {	// position specified?
    fl_set_form_position(fr1->fr1, winPosX,	// yup, adjust top form
			 winPosY);		// size and change
    placementMethod = FL_PLACE_POSITION;	// placement method
  }
  if ((status & WidthValue) &&			// size specified?
      (status & HeightValue))
  fl_set_form_size(fr1->fr1, winWidth,		// yes, adjust top form
		   winHeight);
}


main (int argc, char **argv)
{
  String progname, choice;
  choice = (String) getenv("PRINTER");

  colorSqueeze();				// reduce colormap usage
  if (choice && *choice)			// initialize default printer
    strncpy(printerName, choice, 32);
  fl_initialize(&argc, argv, "Xlpq",		// initialize forms GUI
	  	options, sizeof(options)/
		  sizeof(FL_CMD_OPT));
  fl_get_app_resources(resources,		// parse any resources
		       sizeof(resources)/
			 sizeof(FL_resource));

  if ( *fileName )
    {
      progname = (String) "lpr ";
      if ( *printerName )
	{
	  progname += "-P" + (String) printerName + " ";
	}
      progname += (String) fileName;
      system((const char *) progname);
    }

  fr1 = create_form_fr1();
//  fl_set_browser_fontsize(fr1->browser_1, 11);
  fl_set_browser_fontstyle(fr1->browser_1, FL_FIXED_STYLE);
  fl_set_goodies_font(FL_BOLD_STYLE,12);
  processArgs();				// process arguments

  char inbuf[512];
  String temp, name;
  short empty_printcap = 1;
  ifstream printcap("/etc/printcap", ios::in);
  if( !printcap)
    {
      cout << "Sorry, can't open your printcap file." << endl;
      exit(1);
    }
  while (printcap.getline(inbuf, 512, '\n'))
    {
      if( inbuf[0] == '#' || inbuf[0] == 0) continue;
      temp = inbuf;
      name = temp.before(":");
      if( name.length() == 0 ) continue;
      if( name.contains("|") )
	name = name.before("|");
      while( name.length() != 0 && name[0] == ' ' )
	{
	  name.del(" ");
	}
      if(name.length() == 0) continue;
      while ( temp[temp.length()-1] == '\\')
	{
	  printcap.getline(inbuf, 512, '\n');
	  if( inbuf[0] == '#')
	    temp = "\\";
	  else
	    temp = inbuf;
	}
      if( name.length() != 0 )
	empty_printcap = 0;
      fl_addto_choice(fr1->choice_1, (const char*) name);
      if(name == (String)printerName)
	fl_set_choice_text(fr1->choice_1, (const char*) name);
    }
  printcap.close();
  if( empty_printcap )
    {
      cout << "Sorry, unable to parse your printcap." << endl;
      cout << "Please email your printcap to Christoph.Neerfeld@home.ivm.de" << endl;
      exit(1);
    }
  f_update( (FL_OBJECT *) 0, 0);
  fl_select_browser_line(fr1->browser_1, 1);
  fl_show_form(fr1->fr1, placementMethod, FL_FULLBORDER, "Xlpq");
  fl_set_object_bw(fr1->gr1, -2);
  //  fl_add_signal_callback(SIGALRM, f_timer, (void *) 0);
  //  alarm(10);
  fl_do_forms();
}  
